<?php

/*
 * This file is part of the Jiannei/lumen-api-starter.
 *
 * (c) Jiannei <longjian.huang@foxmail.com>
 *
 * This source file is subject to the MIT license that is bundled
 * with this source code in the file LICENSE.
 */

namespace App\Services\ThirdPlatform\Api;

use App\ImportExport\WuxiLab\WuxiLabExport;
use App\Repositories\Contracts\ThirdApiPlatformRepository;
use App\Services\ThirdPlatform\PlatformAbstractService;
use App\Support\Traits\HttpClientHelpers;


class WuxiApiService extends PlatformAbstractService
{
    use HttpClientHelpers;

    public function __construct(ThirdApiPlatformRepository $thirdApiPlatformRepository)
    {
        parent::__construct($thirdApiPlatformRepository);
        $this->setPlatformName('wuxi_appTec');
        $this->getPlatformInfo();
    }

    /**
     * 批量更新商品信息
     * @param $product
     * @return bool
     */
    public function pushBatchUpdateProduct($product)
    {
        try{
            if($this->checkPlatformStatus()){
                $params = [];
                $params['chemicalProducts'] = $this->mergeApiMetaData($product);

                $response = $this->getPostClient('https://api.uploadcatalog.com/api/v1/update',$params, [
                    $this->platformInfo['platform_params']['username'], $this->platformInfo['platform_params']['password']
                ]);

                return $this->apiResponse($response);
            }
        }catch(\Exception $e){
            return $this->requestError($e);
        }
    }

    public function backgroundLogin()
    {
        if($this->checkPlatformStatus()){
            $response = $this->getPostClient('https://www.uploadcatalog.com/json_login',[
                'username' => $this->platformInfo['platform_params']['username'],
                'password' => $this->platformInfo['platform_params']['password']
            ],null,[
                'cookie' => 'org.springframework.web.servlet.i18n.CookieLocaleResolver.LOCALE=zh_CN; JSESSIONID=B2A024B5414E017F114AE4BCD29A6055'
            ]);

            return json_decode($response,true)['token'];
        }
    }

    public function pushBatchUploadProducts($token)
    {
        if($this->checkPlatformStatus()){
            $params = [];
            $params['companyId'] = '6789';
            $params['type'] = 'PRODUCT_CHEMICAL';
            $params['batchId'] = '7d686e0b-8a08-e9e9-7d1f-916c387dea08';

            $filePath = app(WuxiLabExport::class)->getSavePath('wuxiLab_package_2022_08_08_063609');
            $params['file'] = fopen($filePath.'.csv','r');

            $cookie = 'org.springframework.web.servlet.i18n.CookieLocaleResolver.LOCALE=zh_CN;Auth-Token='.$token.';';

            $response = $this->getPostClient('https://www.uploadcatalog.com/excel/chemical/upload',$params,null,[
                'cookie' => $cookie
            ]);

            return json_decode($response,true)['token'];
        }
    }

    public function pushBatchUpdatePackages($packages,$token)
    {
        try{

            if($this->checkPlatformStatus()){
                $params = [];
                $params['company_id'] = '6789';
                $params['product_type'] = 'CHAMICAL';

                $params = array_merge($params,$packages);

                $cookie = 'org.springframework.web.servlet.i18n.CookieLocaleResolver.LOCALE=zh_CN;Auth-Token='.$token.';';

                $response = $this->getPostClient('https://www.uploadcatalog.com/package/specification/save',$params,null, [
                    'cookie' => $cookie
                ]);

                return $this->apiResponse($response);
            }
        }catch(\Exception $e){
            return $this->requestError($e);
        }
    }

    /**
     * 整理组装 api 数据
     * @param $metaData
     * @return array
     */
    public function mergeApiMetaData($metaData)
    {
        if(empty($metaData)){
           return [];
        }

        $mergeData = [];
        foreach($metaData['rawList'] as $datas){

            $tempData['supplier_product_id'] = $datas['r_code'];
            $tempData['mdl'] = '';
            $tempData['cas'] = $datas['c_cas'];
            $tempData['nu'] = '';
            $tempData['chinese_name'] = $datas['r_cn_name'];
            $tempData['english_name'] = $datas['r_en_name'];
            $tempData['chemicalformula'] = '';
            $tempData['purity'] = $datas['r_level'];
            /*$tempData['usually_ships_days_min'] = '';
            $tempData['usually_ships_within_days'] = '';
            $tempData['backorder_lead_time'] = '';*/
            $tempData['status'] = $datas['r_status'] == '1' ? 'ACTIVE' : 'INACTIVE';

            $tempData['inventorys'][] = [
                'type' => $datas['r_unit'],
                'quantity' => floatval($datas['r_stock']),
                'country' => 'CN',
                'stock_status' => $datas['r_stock'] > 0 ? 'INSTOCK' : 'OUTSTOCK',
            ];

            $tempData['package'] = [];

            if(isset($metaData['packageList'][$datas['r_code']])){
                $package = $metaData['packageList'][$datas['r_code']];
                $pack['unit_description'] = $package['p_pack_unit'];
                $pack['package_status'] = $package['p_status'] == '1' ? 'ACTIVE' : 'INACTIVE';
                $pack['inventorys'] = [
                    'package_quantity' => $package['p_stock'],
                    'stock_status' => $package['p_stock'] > 0 ? 'INSTOCK' : 'OUTSTOK',
                    'country' => 'CN'
                ];

                $pack['prices'] = [
                    'price' => $package['p_price'],
                    'currency' => 'RMB',
                    'status' => 'ACTIVE'
                ];

                $pack['costs'] = [
                    'In_cost' => $package['p_avg_cost'],
                    'In_cost_currency' => 'RMB',
                    'status' => $package['p_stock'] > 0 ? 'ACTIVE' : 'INACTVIE',
                ];

                $pack['wuxiPackagePricings'] = [
                    'discount'=> 0.00,
                    'currency' => 'RMB',
                    'status' => 'INACTIVE'
                ];

                array_push($tempData['package'],$pack);
            }

            array_push($mergeData,$tempData);
            unset($tempData);
        }

        return $mergeData;
    }

    protected function apiResponse($response)
    {
        $decodeResponse = json_decode($response,true);
        if($decodeResponse){
            if(!empty($decodeResponse['successProduct'])){
                return true;
            }

        }
        return false;
    }

    protected function requestError($response)
    {
       return false;
    }
}
