<?php

namespace App\Services;

use App\Repositories\Contracts\ThirdApiPlatformRepository;
use App\Services\Kafka\KafkaService;
use App\Services\ThirdPlatform\Api\ZhenkhApiService;
use App\Support\Facades\SimpleKafka;
use App\Support\Facades\SimpleLogs;

class ZhenKhService
{
    public function __construct()
    {
        $this->apiService = (new ZhenkhApiService(app(ThirdApiPlatformRepository::class)));
        $this->rhawnService = app(RhawnChemicalsService::class);
        $this->rhawnOrderService = app(RhawnOrdersService::class);
    }

    /**
     * @上报商品的价格及库存
     * @param $params
     * @return false|void
     */
    public function consumerUpdateReportPriceAndStock($params)
    {
        if(!isset($params['goods']) || empty($params['goods'])){
            return false;
        }
        $goods = $params['goods'];

        try{
            $product = $this->rhawnService->getRawProductsThroughPid($goods['zg_r_pid']);
            if($product){
                $this->apiService->updateReportProductPrice([
                    'price' => round(bcmul($product->p_price,0.5,2),1),
                    'price_unit' => $goods['zg_unit'],
                    'sku' => $goods['zg_sku']
                ],$this->getToken());

                $this->apiService->updateReportProductStock([
                    'warehouse' => '上海货仓',
                    'address' => ['0' => $goods['zg_addr']],
                    'qty' => $product->p_stock,
                    'sku' => $goods['zg_sku']
                ],$this->getToken());
            }

        }catch (\Throwable $exception) {
            $this->apiService->throwableAfter([
                'platform_name' => $this->apiService->getPlatformInfo('platform_name'),
                'queue_params' => $goods,
                'fail_content' => $exception->getMessage(),
                'queue_reset' => time() + 86400 * 10
            ]);
        }
    }

    /**
     * @更新供应商商品
     * @throws \Throwable
     */
    public function getUpdateSupplierGoods()
    {
        try{
            $page = 1;
            $sendContent = [];
            while(true){
                $pageSize = $this->apiService->getPlatformInfo()['platform_params']['page_size'];
                $goodsList = $this->apiService->getUpdateGoods($page,$pageSize,$this->getToken());
                if($goodsList && count($goodsList['list']) > 0){
                    $sendContent = [
                        'params' => ['goods' => $goodsList['list']],
                        'consumer' => __CLASS__,
                        'method' => 'consumerBatchUpdateGoodsList'
                    ];
                    SimpleKafka::produerSend($sendContent);
                }

                $page ++;
                if($page > $goodsList['total_page']){
                    break;
                }
            }

        }catch(\Throwable $exception){
            $this->apiService->throwableAfter([
                'platform_name' => $this->apiService->getPlatformInfo('platform_name'),
                'queue_params' => $sendContent,
                'fail_content' => $exception->getMessage(),
                'queue_reset' => time() + 86400 * 10
            ]);
            throw $exception;
        }
    }

    /**
     * @获取订单列表
     * @throws \Throwable
     */
    public function getUpdateSupplierOrders()
    {
        try{
            $page = 1;
            $sendContent = [];
            $postParams = [
                'start_time' => date('Y-m-d H:i:s',(time() - 86400)),
                'end_time' => date('Y-m-d H:i:s',strtotime(date('Y-m-d',time())." 23:59:59")),
                'page_size' => 0,
                'page_no' => 0,
            ];
            while(true){
                $pageSize = $this->apiService->getPlatformInfo()['platform_params']['page_size'];
                $postParams['page_size'] = $pageSize;
                $postParams['page_no'] = $page;
                $orderList = $this->apiService->getUpdateOrders($postParams,$this->getToken());
                if($orderList && count($orderList['list']) > 0){
                    foreach($orderList['list']['data'] as $order){
                        if($order['orderStatus'] == 0){
                            SimpleKafka::produerSend([
                                'params' => ['orders' => $order],
                                'consumer' => __CLASS__,
                                'method' => 'consumerBatchUpdateOrdersList'
                            ]);
                        }
                    }
                }

                $page ++;
                if($page > $orderList['total_page']){
                    break;
                }
            }

        }catch(\Throwable $exception){
            $this->apiService->throwableAfter([
                'platform_name' => $this->apiService->getPlatformInfo('platform_name'),
                'queue_params' => $sendContent,
                'fail_content' => $exception->getMessage(),
                'queue_reset' => time() + 86400 * 10
            ]);
            throw $exception;
        }
    }


    /**
     * 消费批量更新商品
     * @param $params
     */
    public function consumerBatchUpdateGoodsList($params)
    {
        if(!isset($params['goods']) || empty($params['goods'])){
            return false;
        }

        foreach($params['goods'] as $goods) {
            try {
                $result = $this->rhawnService->updateZhenkhGoods($goods);
                if(!is_null($result)){
                    //更新商品的价格和库存
                    SimpleKafka::produerSend([
                        'params' => ['goods' => $result],
                        'consumer' => __CLASS__,
                        'method' => 'consumerUpdateReportPriceAndStock'
                    ]);
                }

            } catch (\Throwable $exception) {
                $this->apiService->throwableAfter([
                    'platform_name' => $this->apiService->getPlatformInfo('platform_name'),
                    'queue_params' => $goods,
                    'fail_content' => $exception->getMessage(),
                    'queue_reset' => time() + 86400 * 10
                ]);
            }
        }
    }

    /**
     * @更新订单信息
     * @param $params
     * @return false|void
     */
    public function consumerBatchUpdateOrdersList($params)
    {
        if(!isset($params['orders']) || empty($params['orders'])){
            return false;
        }

        $order = $params['orders'];
        try {
            if($order['orderStatus'] == 0){
                $orderDetail = $this->apiService->getUpdateOrdersDetail($order['purchaseOrderId'],$this->getToken());
                if($orderDetail){
                    $result = $this->rhawnOrderService->updateZhenkhOrders($order,$orderDetail);
                }
            }

        } catch (\Throwable $exception) {
            $this->apiService->throwableAfter([
                'platform_name' => $this->apiService->getPlatformInfo('platform_name'),
                'queue_params' => $order,
                'fail_content' => $exception->getMessage(),
                'queue_reset' => time() + 86400 * 10
            ]);
        }
    }

    /**
     * @ 订单发货
     * @param null $deliveryId
     */
    public function sendDeliveryOrder($deliveryId)
    {
        try{

            $order = $this->rhawnOrderService->getSorderDeliveryDetail($deliveryId);
            $expressArray = $this->apiService->getPlatformDataEntries('zkh_order_express','data_values');
            if(empty($expressArray)){
                $expressList = $this->apiService->expressLogistics($this->getToken());
                if($expressList){
                    foreach($expressList as $express){
                        $tempExpress = [];
                        $tempExpress['id'] = $express['id'];
                        $tempExpress['expressCode'] = $express['expressCode'];
                        $tempExpress['expressName'] = $express['expressName'];
                        $tempExpress['status'] = $express['status'];
                        $tempExpress['logisticsLinkType'] = $express['logisticsLinkType'];
                        $expressArray[] = $tempExpress;
                    }
                    $this->apiService->storePlatformDataEntries('zkh_order_express',$expressArray);
                }
            }

            //$deliveryOrderPost = [];
            //$deliveryOrderPost['deliveryTime']

            //获取发货单的详情
            $deliveryOrder = $this->apiService->sendDeliveryOrder($orderNo,$this->getToken());
            //确认发货
            $this->apiService->ackDeliveryOrder($order,$deliveryOrder,$this->getToken());

        }catch(\Throwable $exception){
            SimpleLogs::writeLog($exception->getMessage(),'sendDeliveryOrder', 'error');
        }
    }

    /**
     * @获取订单送货单pdf
     * @param $deliveryCode
     * @return mixed|void
     */
    public function getDeliveryOrderPdf($deliveryCode)
    {
        $pdf = $this->apiService->getPlatformDataEntries('zkh_order_delivery_pdf','data_values');
        if(!$pdf || empty($pdf)){
            $deliveryInfo = $this->apiService->getDeliveryOrderDetailForPDF($deliveryCode,$this->getToken());
            if($deliveryInfo){
                $this->apiService->storePlatformDataEntries('zkh_order_delivery_pdf',['pdf' => $deliveryInfo]);
                return $deliveryInfo;
            }
        }
        return $pdf['pdf'];
    }

    /**
     * @ 获取api token
     * @return false|mixed
     */
    public function getToken(){
        try{
            $token = $this->apiService->getPlatformDataEntries('zkh_api_token','data_values');
            if(empty($token)){
                $apiToken = $this->apiService->getToken();
                if($apiToken){
                    $apiToken['currentTime'] = time();
                    $this->apiService->storePlatformDataEntries('zkh_api_token',$apiToken);
                    return $apiToken['token'];
                }
            }else{
                //超过有效期，重新请求获取
                if(time() - intval($token['currentTime']) > $token['tokenTtl']){
                    $apiToken = $this->apiService->getToken();
                    if($apiToken){
                        $apiToken['currentTime'] = time();
                        $this->apiService->storePlatformDataEntries('zkh_api_token',$apiToken);
                        return $apiToken['token'];
                    }
                }
                return $token['token'];
            }
        }catch(\Exception $exception){
            SimpleLogs::writeLog($exception->getMessage(), 'token error', 'error');
            return false;
        }
    }
}
